@@ -1,6 +1,6 @@
 /*
  * This file is part of dm-writeboost
- * Copyright (C) 2012-2016 Akira Hayakawa <ruby.wktk@gmail.com>
+ * Copyright (C) 2012-2017 Akira Hayakawa <ruby.wktk@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,12 @@
 #include <linux/dm-io.h>
 #include <linux/dm-kcopyd.h>
 
+/* We use RHEL_RELEASE_VERSION to compile with RHEL/CentOS 7.3's kernel */
+#ifndef RHEL_RELEASE_CODE
+#define RHEL_RELEASE_CODE 0
+#define RHEL_RELEASE_VERSION(a,b) (((a) << 8) + (b))
+#endif
+
 /*----------------------------------------------------------------------------*/
 
 #define SUB_ID(x, y) ((x) > (y) ? (x) - (y) : 0)
@@ -269,8 +275,6 @@ struct wb_device {
 
 	u8 nr_caches_inseg; /* Const */
 
-	struct kmem_cache *buf_1_cachep;
-	mempool_t *buf_1_pool; /* 1 sector buffer pool */
 	struct kmem_cache *buf_8_cachep;
 	mempool_t *buf_8_pool; /* 8 sector buffer pool */
 	struct workqueue_struct *io_wq;
@@ -481,7 +485,7 @@ int prepare_overwrite(struct wb_device *, struct segment_header *, struct metabl
 	do_check_buffer_alignment(buf, #buf, __func__)
 void do_check_buffer_alignment(void *, const char *, const char *);
 
-void bio_endio_compat(struct bio *bio, int error);
+void bio_io_success_compat(struct bio *bio);
 
 /*
  * dm_io wrapper
